# Generated by roxygen2: do not edit by hand

export(approxMaxMin)
export(approxProfileExtrema)
export(bound_profiles)
export(cleanProfileResults)
export(coordProf_UQ)
export(coordinateProfiles)
export(getAllMaxMin)
export(getChangePoints)
export(getPointProportion)
export(getProfileExtrema)
export(getProfileInf_optim)
export(getProfileSup_optim)
export(getSegments)
export(gradKm_dnewdata)
export(obliqueProf_UQ)
export(obliqueProfiles)
export(plotBivariateProfiles)
export(plotMaxMin)
export(plotOblique)
export(plotOneBivProfile)
export(plot_univariate_profiles_UQ)
export(prof_mean_var_Delta)
export(setPlotOptions)
import(microbenchmark)
importFrom(DiceKriging,covMat1Mat2)
importFrom(DiceKriging,covMatrix)
importFrom(DiceKriging,covVector.dx)
importFrom(DiceKriging,km)
importFrom(DiceKriging,predict.km)
importFrom(DiceKriging,simulate)
importFrom(KrigInv,integration_design)
importFrom(MASS,Null)
importFrom(RColorBrewer,brewer.pal)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,cairo_pdf)
importFrom(grDevices,dev.off)
importFrom(grDevices,gray.colors)
importFrom(grDevices,heat.colors)
importFrom(grDevices,pdf)
importFrom(grDevices,terrain.colors)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,contour)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(lhs,maximinLHS)
importFrom(methods,is)
importFrom(pGPx,grad_kweights)
importFrom(pGPx,krig_weight_GPsimu)
importFrom(pGPx,optim_dist_measure)
importFrom(quantreg,predict.rq)
importFrom(quantreg,rq)
importFrom(rcdd,d2q)
importFrom(rcdd,lpcdd)
importFrom(rcdd,makeH)
importFrom(rcdd,q2d)
importFrom(rcdd,scdd)
importFrom(splines,bs)
importFrom(stats,as.formula)
importFrom(stats,deriv)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,smooth.spline)
importFrom(utils,combn)
importFrom(utils,modifyList)
